#Programmer: Ryan Nickell   p0z3r@users.sourceforge.net
#Date: 12/29/2004
#Filename: skQstat.py
#Version: 0.3
#
#Requirements:  Qt: 3.1.1
#		KDE: 3.1.3
#		superkaramba: 0.35
#		kdialog 0.9
#
#    Copyright 2004 Ryan Nickell
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

##
# Imports
###############################################################################
import karamba
import qstat
from xml.parsers.expat import ExpatError

##
# Constants
###############################################################################

##
# Example server that will work and provide a response
# uncomment these two lines and comment out the other two
#########################################################
#type = 'rws'
#host = '67.19.80.148:27960'

type = ''
host = ''


textHandler = 0
headerHandler = 0
headerFontSize = 11
playerFontSize = 10
textColor = 'blue'
headerColor = 'white'
textShadow = 1
headerShadow = 1
font = 'Arial'
originalBackgroundHeight = 0
propertiesMeter = 0
ipAddrMeter = 0
typedict =  {
            'Quake server':'qs',
            'Hexen II server':'h2s',
            'QuakeWorld server':'qws',
            'HexenWorld server':'hws',
            'Quake II server':'q2s',
            'Quake III: Arena server':'q3s',
            'Doom 3 server':'dm3s',
            'Half-Life 2 server':'hl2s',            
            'Return to Castle Wolfenstein or Enemy Territory server':'rws',
            'Star Trek: Elite Force server':'efs',
            'Jedi Knight: Jedi Academy server':'jk3s',
            'Unreal Tournament 2003 server':'ut2s',
            'Unreal server':'uns',
            'Half-Life server':'hls',
            'Sin server':'sns',
            'Shogo: Mobile Armor Division server':'sgs',
            'Tribes server':'tbs',
            'BFRIS server':'bfs',
            'Kingpin server':'kps',
            'Heretic II server':'hrs',
            'Soldier of Fortune server':'sfs',
            'Gamespy Protocol server':'gps',
            'Tribes 2 server':'t2s',
            'Descent3 Gamespy Protocol server':'d3g',
            'Descent3 server':'d3s',
            'Descent3 PXO protocol server':'d3p',
            'Ghost Recon server':'grs',
            'All Seeing Eye Protocol server':'eye',
            'Gamespy V2 Protocol server':'gs2',
            'Ravenshield server':'rss',
            'Savage server':'sas',
            'FarCry server':'fcs',
            'QuakeWorld Master server':'qwm',
            'Quake II Master server':'q2m',
            'Quake III Master server':'q3m',
            'Doom 3 Master server':'dm3m',
            'Return to Castle Wolfenstein Master server':'rwm',
            'Star Trek: Elite Force server':'efm',
            'Jedi Knight: Jedi Academy Master server':'jk3m',
            'Half-Life Master server':'hlm',
            'Tribes Master server':'tbm',
            'Gamespy Master server':'gsm',
            'Tribes 2 Master server':'t2m',
            'Descent3 Master (PXO) server':'d3m',
            'Steam Master server':'stm',
	    'UT2004 server':'ut2004s',
	    'UT3 server':'gs4'
            }

##
# Custom Functions
###############################################################################
def getPlayers(type, host):
    """ This returns a list of the current players on the server
    """
    listPlayers = []
    application = qstat.Application()
    try:
        server = application.run(type,host)
    except ExpatError:
        return "SERVER DOWN"
    except AssertionError:
        return "SERVER DOWN"
    for player in server.players:
        listPlayers.append((player.name,player.score))
    listPlayers = sortPlayers(listPlayers)
    return listPlayers

def sortPlayers(list):
    """ This sorts the players by descending score, ie. more Frags are at the
    top of the list.
    """
    for i in range(len(list)):
        for j in range(len(list)-1):
            if list[j][1] < list[j+1][1]:
                temp = list[j]
                list[j] = list[j+1]
                list[j+1] = temp
    return list

def getHeader(type, host):
    """ This returns the header which is the server name and 
    map name for now."""
    application = qstat.Application()
    try:
        server = application.run(type,host)
    except ExpatError:
        return 'Server: Down'
    except AssertionError:
        return 'Server: Down'
    try:
        header = 'Server:\t '+server.name+'\n'
        #header += 'Number of players: \t'+str(server.numPlayers)+'\n'
        header += 'Map:\t '+server.map
    except:
        header = 'Server: Down'
    return header

def verifyIp(ipAddrHost):
    splitHost = ipAddrHost.split(":")
    if len(splitHost) in [1,2]:
        #verify the ip address
        ipAddr = splitHost[0]
        octets = ipAddr.split(".")
        if len(octets) != 4:
            return False
        for octet in octets:
            try:
                if not (int(octet) >= 0 and int(octet) <= 255):
                    return False
            except ValueError:
                return False
        if len(splitHost) == 2:
            port = splitHost[1]
            #verify the port now
            try:
                int(port)
            except ValueError:
                # It wasn't an integer type
                return False
    else:
        # wrong length after the split
        return False
    return True

def prettyText(widget, size, font, textname, fontColor, shadow):
    """This is a utility function for converting text to something more usable.
    You can add more color support by filling in your own hex values for color 
    codes.
    """
    karamba.changeTextSize(widget, textname, size)
    karamba.changeTextFont(widget, textname, font)
    if(fontColor == "white"):
        karamba.changeTextColor(widget, textname, 255, 255, 255)
    elif(fontColor == "white"):
        karamba.changeTextColor(widget, textname, 255, 255, 255)
    elif(fontColor == "blue"):
        karamba.changeTextColor(widget, textname, 43, 114, 190)
    elif(fontColor == "red"):
        karamba.changeTextColor(widget, textname, 255, 0, 0)
    elif(fontColor == "green"):
        karamba.changeTextColor(widget, textname, 0, 255, 0)
    elif(fontColor == "yellow"):
        karamba.changeTextColor(widget, textname, 255, 255, 0)
    elif(fontColor == "purple"):
        karamba.changeTextColor(widget, textname, 127, 0, 127)
    elif(fontColor == "tan"):
        karamba.changeTextColor(widget, textname, 127, 127, 0)
    elif(fontColor == "custom"):
        #4C4F7A - 0100 1101 0100 1111 0111 1010
        #       - 64+8+4+1 = 77 64+15 = 79 64+32+16+8+2 = 122
        #30324D - 0011 0000 0011 0010 0100 1110
        #       - 32+16 = 48 48+2 = 50 64+14 = 78
        karamba.changeTextColor(widget, textname, 48, 50, 78)
    else:
        karamba.changeTextColor(widget, textname, 0,0,0)
    karamba.changeTextShadow(widget, textname, shadow)

##
# Superkaramba Callbacks
###############################################################################
def initWidget(widget):
    global propertiesMeter,type,host,background,originalBackgroundHeight
    temp = karamba.readConfigEntry(widget,"type")
    if temp:
        type = temp.strip()
    temp = karamba.readConfigEntry(widget,"host")
    if temp and verifyIp(temp):
        host = temp.strip()	
    propertiesMeter = karamba.createImage(widget,3,-5,'images/green.png')
    karamba.addImageTooltip(widget,propertiesMeter,'Click here to change the server')
    karamba.attachClickArea(widget,propertiesMeter)
    background = karamba.createBackgroundImage(widget,0,30,'images/background.png')
    originalBackgroundHeight = karamba.getImageHeight(widget,background)
    karamba.resizeImage(widget,background,240,karamba.getImageHeight(widget,background))
    karamba.redrawWidgetBackground(widget)
    widgetUpdated(widget)
    karamba.redrawWidget(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    global type, host,textHandler,headerHandler,headerFontSize,playerFontSize
    global textColor, headerColor, font, propertiesButton, textShadow, headerShadow
    global background
    textHeader = getHeader(type,host)
    try:
        karamba.deleteText(widget,textHandler)
        karamba.deleteText(widget,headerHandler)
    except:
        pass
    headerHandler = karamba.createText(widget,16,55,310,20,textHeader)
    prettyText(widget, headerFontSize, font, headerHandler, headerColor, headerShadow)

    players = getPlayers(type,host)
    text = 'Score:\tPlayer:\n'
    if players != "SERVER DOWN":
        for item in players:
            text += str(item[1])
            text += '\t'
            text += str(item[0])
            text += '\n'

    textHandler = karamba.createText(widget,55,85,310,20,text)
    prettyText(widget, playerFontSize, font, textHandler, textColor, textShadow)
    # Resize to match the amount of users
    if len(players) > 7 and (originalBackgroundHeight + (14*(len(players)-7)) <= 310):
        karamba.resizeImage(widget,background,230,originalBackgroundHeight+(14*(len(players)-7)))
        karamba.redrawWidgetBackground(widget)
    # Max out at 300 in height
    elif originalBackgroundHeight + (14*(len(players)-7)) > 310:
        karamba.resizeImage(widget,background,230,310)
        karamba.redrawWidgetBackground(widget)
    # At minimum always show the height of the background as 149 (default size)
    else:
        karamba.resizeImage(widget,background,230,originalBackgroundHeight)
        karamba.redrawWidgetBackground(widget)
    karamba.redrawWidget(widget)
    propertiesMeter = karamba.createImage(widget,222,17,'images/list.png')

#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass


#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    global propertiesMeter,ipAddrMeter,typedict
    global kdialogPid,kIpPid
    if meter == propertiesMeter:
        values = typedict.keys()
        values.sort()
        kdialog = ["kdialog", "--title", "Server Type", "--radiolist", "Pick type of game server to query:"]
        for i in range(len(values)):
            kdialog.append(values[i])
            kdialog.append(values[i])
            if i == 0:
                kdialog.append("on")
            else:
                kdialog.append("off")
        kdialogPid = karamba.executeInteractive(widget,kdialog)

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
    global kdialogPid,kIpPid
    global type,host,typedict,prevType
    if pid == kdialogPid:
        output = output.strip()
        if typedict.has_key(output):
            prevType = type
            type = typedict[output]
            karamba.writeConfigEntry(widget, "type",type)
        else:
            #do not change it?
            print "Error with type entered: ",type
        kdialog = ["kdialog","--title","IP Address","--inputbox","Enter the IP address of the server:\n(for example: 192.168.0.1 or 192.168.0.1:27960)"]
        kIpPid = karamba.executeInteractive(widget,kdialog)
    elif pid == kIpPid:
        output = output.strip()
        if verifyIp(output):
            host = output
            karamba.writeConfigEntry(widget, "host", host)
        else:
            #reset our type back because of failure on host
            type = prevType
        widgetUpdated(widget)

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass


#This is called when your widget is closed.  You can use this to clean
#up open files, etc.  You don't need to delete text and images in your
#theme.  That is done automatically.  This callback is just for cleaning up
#external things.  Most people don't need to put anything here.
def widgetClosed(widget):
    pass

# This will be printed when the widget loads.
print "Loaded skQstat game server monitor!"

